#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */

#include "messages.h"

int sendFWDModeMsg();
int sendFWDMsg();
int pollTTPStatus();
int sendSTOPMsg();

unsigned char keyword[2];
int fd;

int main()
{
  int stop = 0, len;
  char *command;
  
  fd = open_port();
  printf("port opend \n");
  command = (char *)malloc(100);
  printf("\ncommand#");
  while(stop == 0)
    {
      command = fgets( command, 10, stdin );
      if(command != NULL)
	{
	  len = strlen(command);
	  command[len - 1] = '\0';
	  if(strcmp("play", command) == 0)
	    {	      
	      sendFWDModeMsg();
	      sendFWDMsg();
	      pollTTPStatus();
	    }
	  else if(strcmp("stop", command) == 0)
	    {
	      sendSTOPMsg();
	    }
	  else if(strcmp("quit", command) == 0)
	    {
	      printf(" quit ... \n");
	      stop = 1;
	    }
	}
      printf("\ncommand#");
    }

  close(fd);
  return 0;
}


int sendFWDModeMsg()
{
  unsigned char checksum =0, msg[5], ACKMSG[3], NAKMSG[5];
  char recvbuff[256];
  int i = 0, errmsg = 0, acknmsg, msglen;
  memcpy(keyword,&FWD,2);
  msg[0] = keyword[1];
  msg[1] = keyword[0];
  msg[2] = 0x1;
  msg[3] = 0x0;
  checksum = msg[0] + msg[1] + msg[2] + msg[3];
  msg[4] = checksum;
  
  printf("sending FWD Mode Msg  ... ");  
  put_bytes(fd, msg, 5);
  printf(" done. \n");
  
  ACKMSG[0] = 0x00;
  ACKMSG[1] = 0x02;
  ACKMSG[2] = 0x02;
  
  NAKMSG[0] = 0x08;
  NAKMSG[1] = 0x03;
  NAKMSG[2] = 0x01;
    
  while(1)
    {
      recvbuff[i] = get_byte(fd);
      if(recvbuff[i] == -1) {
	printf("error reading : count %d \n", i);
	exit(1);
      }
      
      if(i == 0)
	{
	  if(recvbuff[i] == ACKMSG[0])
	    {
	      acknmsg = 1;
	      msglen = 3;
	    }
	  else if(recvbuff[i] == NAKMSG[0])
	    {
	      errmsg = 1;
	      msglen = 5;
	    }
	}
      i++;
      if(msglen == i)
	break;
    }
  
  if(msglen == 3)
    {
      if( (ACKMSG[1] != recvbuff[1]) & (ACKMSG[2] != recvbuff[2]))
	{
	  printf("wrong ACK received - msg: %X %X %X \n",recvbuff[0],recvbuff[1],recvbuff[2]); 
	  exit(1);
	}
    }
  if(msglen == 5)
    {
      printf(" NAK received - msg error : %X \n", recvbuff[3]);
      exit(1);
    }
  return 1;
}

int sendFWDMsg()
{
  unsigned char checksum =0, msg[3], ACKMSG[3], NAKMSG[5];
  char recvbuff[256];
  int i = 0, errmsg = 0, acknmsg, msglen;
  memcpy(keyword,&FORWARD,2);
  msg[0] = keyword[1];
  msg[1] = keyword[0];
  
  checksum = msg[0] + msg[1];
  msg[2] = checksum;

  printf("sending FWD Msg  ... ");  
  put_bytes(fd, msg, 3);
  printf(" done. \n");

  ACKMSG[0] = 0x00;
  ACKMSG[1] = 0x02;
  ACKMSG[2] = 0x02;
  
  NAKMSG[0] = 0x08;
  NAKMSG[1] = 0x03;
  NAKMSG[2] = 0x01;
    
  while(1)
    {
      recvbuff[i] = get_byte(fd);
      if(recvbuff[i] == -1) {
	printf(" error reading : count %d \n", i);
	exit(1);
      }
      
      if(i == 0)
	{
	  if(recvbuff[i] == ACKMSG[0])
	    {
	      acknmsg = 1;
	      msglen = 3;
	    }
	  else if(recvbuff[i] == NAKMSG[0])
	    {
	      errmsg = 1;
	      msglen = 5;
	    }
	}
      i++;
      if(msglen == i)
	break;
    }
  
  if(msglen == 3)
    {
      if( (ACKMSG[1] != recvbuff[1]) & ACKMSG[2] != recvbuff[2])
	{
	  printf("FWD Msg: wrong ACK received - msg: %X %X %X \n",recvbuff[0],recvbuff[1],recvbuff[2]); 
	  exit(1);
	}
    }
  if(msglen == 5)
    {
      printf("FWD Msg: NAK received - msg error : %X \n", recvbuff[3]);
      exit(1);
    }
  return 1;
}

int pollTTPStatus()
{
  unsigned char checksum =0, msg[3], TTPRETMSG[3], NAKMSG[5];
  char recvbuff[256];
  int i = 0, errmsg = 0, ttpmsg, msglen, brkrecv = 0;
  unsigned char SERVOLOCK = 0x80, CHECK, recvmsg[256];
  
  memcpy(keyword,&TTPSTATUS,2);
  msg[0] = keyword[1];
  msg[1] = keyword[0];
  
  checksum = msg[0] + msg[1];
  msg[2] = checksum;
  TTPRETMSG[0] = 0x3C;
  TTPRETMSG[1] = 0x01;
  TTPRETMSG[2] = 0x04;
  
  NAKMSG[0] = 0x08;
  NAKMSG[1] = 0x03;
  NAKMSG[2] = 0x01;
  
  checksum = 0;
  printf("sending TTP Msg  ... done. ");  
  while(1)
    {
      put_bytes(fd, msg, 3);
      while(brkrecv == 0)
	{
	  recvbuff[i] = get_byte(fd);
	  if(recvbuff[i] == -1) {
	    printf(" error reading : count %d \n", i);
	    exit(1);
	  }
	  recvmsg[i] = (unsigned char)recvbuff[i];
	  checksum += recvmsg[i];
	  if(i == 0)
	    {
	      if(recvmsg[i] == TTPRETMSG[0])
		{
		  ttpmsg = 1;
		  msglen = 8;
		}
	      else if(recvmsg[i] == NAKMSG[0])
		{
		  errmsg = 1;
		  msglen = 5;
		}
	    }
	  i++;
	  if(msglen == i){
	    checksum = checksum - recvmsg[i-1];
	    brkrecv = 1;
	  }
	}
      if((ttpmsg) & (msglen==8) & (checksum==recvmsg[7]))
	{
	  CHECK = recvmsg[4] & SERVOLOCK;
	  if(CHECK == SERVOLOCK)
	    break;
	  else
	    {
	      msglen = 0;
	      i=0;
	      ttpmsg = 0;
	      brkrecv = 0;
	      checksum = 0;
	    }
	}
      else if((errmsg) & (msglen == 5))
	{
	  printf("TTP Msg: NAK received - msg error : %X \n", recvmsg[3]);
	  exit(1);
	}
    }
}

int sendSTOPMsg()
{
  unsigned char checksum =0, msg[3], ACKMSG[3], NAKMSG[5];
  char recvbuff[256];
  int i = 0, errmsg = 0, acknmsg, msglen;
  memcpy(keyword,&STOP,2);
  msg[0] = keyword[1];
  msg[1] = keyword[0];
  
  checksum = msg[0] + msg[1];
  msg[2] = checksum;

  printf("sending STOP Msg  ... ");  
  put_bytes(fd, msg, 3);
  printf(" done. \n");

  ACKMSG[0] = 0x00;
  ACKMSG[1] = 0x02;
  ACKMSG[2] = 0x02;
  
  NAKMSG[0] = 0x08;
  NAKMSG[1] = 0x03;
  NAKMSG[2] = 0x01;
    
  while(1)
    {
      recvbuff[i] = get_byte(fd);
      if(recvbuff[i] == -1) {
	printf(" error reading : count %d \n", i);
	exit(1);
      }
      
      if(i == 0)
	{
	  if(recvbuff[i] == ACKMSG[0])
	    {
	      acknmsg = 1;
	      msglen = 3;
	    }
	  else if(recvbuff[i] == NAKMSG[0])
	    {
	      errmsg = 1;
	      msglen = 5;
	    }
	}
      i++;
      if(msglen == i)
	break;
    }
  
  if(msglen == 3)
    {
      if( (ACKMSG[1] != recvbuff[1]) & ACKMSG[2] != recvbuff[2])
	{
	  printf("STOP Msg: wrong ACK received - msg: %X %X %X \n",recvbuff[0],recvbuff[1],recvbuff[2]); 
	  exit(1);
	}
    }
  if(msglen == 5)
    {
      printf("STOP Msg: NAK received - msg error : %X \n", recvbuff[3]);
      exit(1);
    }
  return 1;
}
